import type {UGridRow, UGridColumnDefinition, SubElementProps} from '@univention/univention-veb';

export interface LabeledValue<T> {
  value: T;
  label: string;
}

// This should be coming from the apiTypes, but is not implemented yet
export type SortingOrder = 'asc' | 'desc' | 'none';
export type Formatter = 'str' | 'unknown';

export interface Column {
  label: string;
  attribute: string;
  // sorting: SortingOrder;
  formatter?: string;
}

export type GlobalAction = 'add';
export interface ListViewConfig {
  url: string;
  allowedGlobalActions: GlobalAction[];
  searchFormOptions: {
    schools: LabeledValue<string>[];
    idiSchoolyears: LabeledValue<string>[];
    idiSchoolyearPreselect: string;
    divisClasstype: LabeledValue<string>[];
    activationStates: LabeledValue<string>[];
    isLmsGroup: LabeledValue<string>[];
    isIservGroup: LabeledValue<string>[];
  };
  columns: Column[];
}

export interface AddViewConfig {
  pages: Page[];
}

export type Access = 'none' | 'read' | 'write';
export type ContextAction = 'delete' | 'edit' | 'toggleLMS' | 'toggleIServ';


export type SchoolGroupType = 'school_class' | 'workgroup';
export interface Attribute {
  value?: unknown;
  access: Access;
}
export interface GroupListResponseModel {
  id: string;
  allowedActions: ContextAction[];
  schoolGroupType: SchoolGroupType;
  attributes: Record<string, Attribute>;
}

export interface FieldInputText {
  type: 'UInputText';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    hint?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';
  };
}
export interface FieldInputDate {
  type: 'UInputDate';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    hint?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';
  };
}
export interface FieldInputPassword {
  type: 'UInputPassword';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    hint?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';
  };
}
export interface FieldInputCheckbox {
  type: 'UInputCheckbox';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    hint?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';
  };
}
export interface FieldSelect {
  type: 'USelect';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    hint?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';
    options: {label: string; value: string}[];
  };
}
export interface FieldComboBox {
  type: 'UComboBox';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    hint?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';
    options: {label: string; value: string}[];
  };
}
export interface FieldMultiSelect {
  type: 'UMultiSelect';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    hint?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';
    columns: UGridColumnDefinition[];
    rows: UGridRow[];
  };
}
export interface FieldMultiObjectSelect {
  type: 'UMultiObjectSelect';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    hint?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';

    initialRows: UGridRow[];
    columns: UGridColumnDefinition[];

    searchStandby: boolean;
    searchRows: UGridRow[];
    searchElements: SubElementProps[];
    searchColumns: UGridColumnDefinition[];
    searchCallback?: (values: Record<string, unknown>) => void;
  };
}
export interface FieldInputClassified {
  type: 'UInputClassified';
  props: {
    name: string;
    label: string;
    description?: string;
    access?: 'none';
  };
}
export type Field =
  FieldComboBox |
  FieldInputCheckbox |
  FieldInputClassified |
  FieldInputDate |
  FieldInputPassword |
  FieldInputText |
  FieldMultiSelect |
  FieldMultiObjectSelect |
  FieldSelect;


export interface Fieldset {
  label: string;
  name: string;
  rows: (Field | Field[])[];
}

export interface Page {
  label: string;
  name: string;
  fieldsets: Fieldset[];
}

export type FormValues = Record<string, unknown>;
export type DetailAction = 'save';
export interface SchoolClassDetailResponseModel {
  url: string;
  allowedActions: DetailAction[];
  pages: Page[];
  values: FormValues;
  usersSearchParams: FormValues;
}
export interface WorkGroupDetailResponseModel {
  url: string;
  allowedActions: DetailAction[];
  pages: Page[];
  values: FormValues;
  usersSearchParams: FormValues;
}

export type ToggleActiveAction = 'activate' | 'deactivate';
export type ToggleActiveProp = 'lms' | 'iserv';
